# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import collections
import re


def extract_normalized_response_name(response_name):
    """ Remove custom parameters from a response file name.
        E.G.
        "bar_{business_id#foo}_{mtb_id}" -> "bar_{business_id}_{mtb_id}"

        :param response_name: a json mock response filename
        :return: cleaned-up response name
    """
    return re.sub(r'{(\w+)(?:#\w+)?}', r'{\1}', response_name)


def extract_arg_pattern_from_response_name(response_name):
    """ Extract the argument pattern from a filename.
        This will be used to perform view matching

        :param response_name: a json mock response filename
        :return: argument pattern, a dictionary containing the arguments
            defined by the file name and their value (or None)
    """
    arguments = re.findall(r'{(.+?)}', response_name)
    args_pattern = {}

    for argument in arguments:
        splits = re.split(r'#', argument)
        if len(splits) == 1:
            param = splits[0]
            value = None
        elif len(splits) == 2:
            param = splits[0]
            value = splits[1]
        else:
            raise ValueError(
                "{0} has too many arguments".format(response_name)
            )

        args_pattern[param] = value

    return args_pattern


def extract_arg_pattern_from_query_args(query_args):
    """ Extract the argument pattern from query_args

        :param query_args:
        :return: argument pattern, a dictionary containing the arguments
            defined by the file name and their valu
    """
    arguments = re.findall(r'{(.+?)}', query_args)
    args_pattern = {}

    for argument in arguments:
        splits = re.split(r'#', argument)
        if len(splits) == 2:
            param = splits[0]
            value = splits[1]
        else:
            raise ValueError(
                "Query is not correctly defined/extracted: {0}".format(query_args)
            )

        args_pattern[param] = value

    return args_pattern


class Operation(collections.namedtuple(
    'OperationBase', ['response_name', 'http_verb', 'query_args']
)):
    def __new__(cls, response_name, http_verb, query_args=None):
        return super(Operation, cls).__new__(
            cls,
            response_name=response_name,
            http_verb=http_verb,
            query_args=query_args,
        )


def make_operation_from_path(path, http_verb):
    """ Make an operation from a url path and the associated http_verb

    Sample result:
    '/foo/bar/', 'GET'
        -> Operation('foo_bar', 'GET')

    :param path: url path
    :param http_verb: POST, PUT, GET ...
    :return: (response_name, http_verb) tuple
    """
    name = path.strip('/').replace('/', '_')
    return Operation(name, http_verb.upper())


def _filename_regex_match(filename):
    """ Main function to match filename with the core regex to extract all the information from
     the file names

    :param filename: the name of the response file, without extension
    :return: re.MatchObject or None if the filename is not valid
    """
    return re.match(
        r'^(?P<response_name>.+?)(_query(?P<query_args>(?:{[^{}#]+#[^{}#]+})+))?_response\.(?:(?P<http_code>\d+)\.)?(?P<http_verb>[A-Z]+)$',  # noqa
        filename,
    )


def extract_operation_and_response_code_from_filename(filename):
    """ Extract from the filename, the operation (name + http verb) and eventual
    response http code defaulting to 200 if absent.

    Sample result:
    'foo_{foo_id#404}_v1_response.404.GET'
        -> Operation('foo_{foo_id#404}_v1', 'GET'), 404

    :param filename: the name of the response file, without extension
    :return: (response_name, http_verb) tuple and http_code int, or (None, None) if unvalid
    """
    match = _filename_regex_match(filename)
    if match:
        response_name = match.group('response_name')
        http_verb = match.group('http_verb')
        http_code = match.group('http_code') or 200
        http_code = int(http_code)
        query_args = match.group('query_args')

        return Operation(response_name, http_verb, query_args=query_args), http_code
    else:
        return None, None


def check_file_is_valid(filename):
    """

    :param filename: the name of the response file, without extension
    :return: True if filename is a valid mock file
    """
    return _filename_regex_match(filename) is not None


def norm_operation(operation):
    """ Return a normalized version of an operation, where the name is
    normalized, http_verb is standardized and query arguments are removed

    :param operation: Operation
    :return: normalized operation
    """
    normed_name = extract_normalized_response_name(operation.response_name)
    return Operation(normed_name, operation.http_verb.upper(), query_args=None)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

from collections import Counter
from collections import defaultdict
from itertools import chain

import six

from pyramid_mock_server.util import extract_arg_pattern_from_query_args
from pyramid_mock_server.util import extract_arg_pattern_from_response_name
from pyramid_mock_server.util import norm_operation


class _ResponseVariations(object):
    """ Group the different possible response variations and actually performs
    the matching from the parameters
    """

    def __init__(self, mock_responses=None):
        """ Initialize the instance.
        The optional parameter is here to test this object.

        :param mock_responses: List of mock response names
        """
        self._arg_pattern_by_operation = {}
        self._mock_response_by_operation = {}

        if mock_responses:
            for mock_response in mock_responses:
                self.add_mock_response(mock_response)

    def add_mock_response(self, mock_response):
        arg_pattern = extract_arg_pattern_from_response_name(mock_response.operation.response_name)
        if mock_response.operation.query_args:
            arg_pattern.update(
                extract_arg_pattern_from_query_args(mock_response.operation.query_args)
            )
        self._arg_pattern_by_operation[mock_response.operation] = arg_pattern
        self._mock_response_by_operation[mock_response.operation] = mock_response

    def get_arg_list(self):
        """ Return the list of arguments that is defined by the list of
        filenames put in this object

        :return: list of arguments as an array of strings
        """
        arg_list = {}
        for pattern in six.itervalues(self._arg_pattern_by_operation):
            arg_list.update(pattern)

        return arg_list.keys()

    def generate_string_replacement_dict(self):
        """ Return all the variations as dictionaries that can be used to
        format path strings with the arguments that would lead to call this
        variation

        :return: list of string_replacement dict
        """
        arg_list = self.get_arg_list()
        arg_patterns = self._arg_pattern_by_operation.values()
        default_args_dict = {arg: arg for arg in arg_list}

        string_replacement_dicts = []
        for arg_pattern in arg_patterns:
            string_replacement_dict = default_args_dict.copy()
            string_replacement_dict.update({
                k: v
                for k, v in six.iteritems(arg_pattern) if v
            })
            string_replacement_dicts.append(string_replacement_dict)

        return string_replacement_dicts

    def match_arg_dict(self, arg_dict):
        """ Find the best matching variation for the given argument dictionary

        :param arg_dict: the requests arguments
        :return: best known matching MockResponse
        """

        # If their is only one alternative, return it directly
        if len(self._mock_response_by_operation) == 1:
            return next(six.itervalues(self._mock_response_by_operation))

        # Find the alternative that matches arguments best prioritizing
        # most specific value match over the generic.
        matchs = defaultdict(list)
        for key, value in six.iteritems(arg_dict):
            for operation, pattern in six.iteritems(self._arg_pattern_by_operation):
                if key in pattern and pattern[key] == value:
                    matchs[key].append(operation)

        for key in self.get_arg_list():
            # We only try to match the default pattern if no particular pattern
            # has been found.
            if not matchs[key]:
                for operation, pattern in six.iteritems(self._arg_pattern_by_operation):
                    if pattern.get(key) is None:
                        matchs[key].append(operation)

        # We use Counter to get the most matched name in the previous step
        all_matches = chain.from_iterable(six.itervalues(matchs))
        best_matched_operation = Counter(all_matches).most_common(1)[0][0]
        return self._mock_response_by_operation[best_matched_operation]


class ResponseCollection(object):
    """ Allow to group the different files for the same endpoint by analyzing
    the filename to guess the correct endpoint and the specific arguments for
    which this view should be used
    """

    def __init__(self, mock_responses):
        """
        :param mock_responses: array of read MockResponses
        """
        self._internal_storage = defaultdict(_ResponseVariations)
        for mock_response in mock_responses:
            self.add_mock_response(mock_response)

    def add_mock_response(self, mock_response):
        normed = norm_operation(mock_response.operation)
        self._internal_storage[normed].add_mock_response(mock_response)

    def get_variations(self, response_operation):
        normed_operation = norm_operation(response_operation)
        return self._internal_storage.get(normed_operation, None)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import os
from collections import namedtuple

import jinja2

from pyramid_mock_server.jinja_utils import JSONOverrideExtension
from pyramid_mock_server.jinja_utils import JSONPatchExtension
from pyramid_mock_server.util import extract_operation_and_response_code_from_filename


MockResponse = namedtuple(
    'MockResponse',
    [
        'operation',
        'json_str',
        'http_response_code',
    ],
)


def _make_json_template_loader(mock_responses_directory):
    """ Generate a rendering function that uses jinja in the context of mock_responses_directory
        to render json files.

        :param: mock_responses_directory: path to root responses directory
        :return: loader function, that take a path of the json file one wants to load and returns
            a rendered json string
    """
    env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(mock_responses_directory),
        extensions=[JSONOverrideExtension, JSONPatchExtension]
    )

    def _load_json_template(filepath):
        template_filepath = os.path.relpath(
            filepath,
            start=mock_responses_directory,
        )
        return env.get_template(template_filepath).render()

    return _load_json_template


def load_responses(mock_responses_directory):
    """ Load all sample json files in the given directory that match the naming pattern, ignoring
        all others.

        :param: mock_responses_directory: path to root responses directory
        :return: array of MockResponse
    """
    mock_responses = []
    load_json_template_fn = _make_json_template_loader(mock_responses_directory)

    for root, _, files in os.walk(mock_responses_directory):
        for filename in files:
            response_name, ext = os.path.splitext(filename)
            if ext != ".json":
                continue

            json_filepath = os.path.join(root, filename)
            json_str = load_json_template_fn(json_filepath)

            operation, response_code = extract_operation_and_response_code_from_filename(
                response_name
            )

            if operation is not None and response_code is not None:
                mock_responses.append(MockResponse(
                    operation=operation,
                    json_str=json_str,
                    http_response_code=response_code,
                ))

    return mock_responses



# -*- coding: utf-8 -*-
"""
Import this module to add the mocks views to your pyramid app.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

from pyramid_mock_server.swagger_util import get_swagger20_resources_iterator_from_pyramid_swagger
from pyramid_mock_server.view_maker import setup_routes_views


def includeme(config):
    """
    :type config: :class:`pyramid.config.Configurator`
    """
    settings = config.registry.settings

    custom_view_packages = settings.get('pyramid_mock_server.custom_view_packages')
    responses_path = settings.get('pyramid_mock_server.mock_responses_path')
    excluded_paths = settings.get('pyramid_mock_server.excluded_paths')

    # Read resources:
    resources = settings.get('pyramid_mock_server.resources', [])

    # Read resources from pyramid_swagger
    if settings.get('pyramid_mock_server.get_resources_from_pyramid_swagger_2_0_schema', False):
        for resource in get_swagger20_resources_iterator_from_pyramid_swagger(config):
            resources.append(resource)

    setup_routes_views(
        config=config,
        responses_path=responses_path,
        resources=resources,
        excluded_paths=excluded_paths,
        custom_view_packages=custom_view_packages,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2014-2017, Yelp, Inc.

import os
from setuptools import setup

base_dir = os.path.dirname(__file__)

setup(
    name='pyramid_mock_server',
    version='1.1.1',
    licence='BSD 3-Clause License',

    description='Create pyramid server that return mocked response from an OpenAPI spec',
    long_description=open(os.path.join(base_dir, 'README.md')).read(),
    url='https://github.com/Yelp/pyramid_mock_server',

    author='Yelp, Inc.',
    author_email='opensource+pyramid_mock_server@yelp.com',

    packages=['pyramid_mock_server'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[
        'jinja2>=2.7',
        'pyramid>=1.4',
        'six',
        'venusian>=1.0',
    ],
    extra_requires={
        'Swagger': ['pyramid_swagger >= 2.3.0'],
    },
)
